package test.jndi.ejb;

import java.util.Properties;
import java.util.StringTokenizer;

import javax.ejb.CreateException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NameNotFoundException;
import javax.naming.NamingException;

import test.jndi.remote.TestRemote;

/**
 * XDoclet-based session bean. The class must be declared public according to the EJB specification.
 * 
 * To generate the EJB related files to this EJB: - Add Standard EJB module to XDoclet project properties - Customize
 * XDoclet configuration for your appserver - Run XDoclet
 * 
 * Below are the xdoclet-related tags needed for this EJB.
 * 
 * @ejb.bean name="RMIClientService" display-name="RMI Client Service Support Interface" description="This service is
 *           used by RMI Server" jndi-name="ejb/RMIClientHome" type="Stateless" view-type="remote"
 * 
 * @ejb.transaction type="Required"
 * 
 */

public class RMIClientEJB implements SessionBean {

    /**
     * This is a reference to the <code>SessionContext</code> for this bean
     */
    protected SessionContext context;

    /**
     * The <code>ejbActivate()</code> method as required by the EJB specification. <br>
     * 
     * The activate method is called when the instance is activated from its passive" state. The instance should acquire
     * any resource that it has released earlier in the <code>ejbPassivate()</code> method. <br>
     * 
     * This method is called with no transaction context.
     * 
     */
    public void ejbActivate() {}

    /**
     * The <code>ejbPassivate()</code> method as required by the EJB specification. <br>
     * 
     * The activate method is called when the instance is activated from its "passive" state. The instance should
     * acquire any resource that it has released earlier in the <code>ejbActivate()</code> method. <br>
     * 
     * This method is called with no transaction context.
     * 
     */
    public void ejbPassivate() {}

    /**
     * The <code>ejbRemove()</code> method as required by the EJB specification. <br>
     * 
     * A container invokes this method before it ends the life of the session object. This happens as a result of a
     * client's invoking a remove operation, or when a container decides to terminate the session object after a
     * timeout. <br>
     * 
     * This method is called with no transaction context.
     * 
     */
    public void ejbRemove() {}

    /**
     * Set the associated session context. The container calls this method after the instance creation. <br>
     * 
     * The enterprise bean instance should store the reference to the context object in an instance variable. <br>
     * 
     * This method is called with no transaction context.
     * 
     * @see javax.ejb.SessionBean#setSessionContext(javax.ejb.SessionContext)
     */
    public void setSessionContext(SessionContext newContext) {
        context = newContext;
    }

    /**
     * An <code>ejbCreate</code> method as required by the EJB specification. <br>
     * 
     * The container calls the instances <code>ejbCreate</code> method whose signature matches the signature of the
     * <code>create</code> method invoked by the client. The input parameters sent from the client are passed to the
     * <code>ejbCreate</code> method. Each session bean class must have at least one <code>ejbCreate</code> method. The
     * number and signatures of a session beans <code>create</code> methods are specific to each session bean class.
     * 
     * @throws CreateException Thrown if the instance could not perform the function requested by the container because
     *             of an system-level error.
     * 
     * @ejb.create-method
     */
    public void ejbCreate() throws CreateException {}

    /**
     * This methods registers a reference to remote RMI object and after second call will fail.
     * 
     * @ejb.interface-method view-type = "remote"
     * @ejb.transaction type = "Never"
     * 
     */
    public void registerRemote(TestRemote remote) throws Exception {
        Context ctx = getOrCreateContext(TestRemote.CONTEXT_PATH);

        ctx.rebind(TestRemote.JNDI_NAME, remote);
    }

    /**
     * This methods registers a reference to remote RMI object and does not fail
     * 
     * @ejb.interface-method view-type = "remote"
     * @ejb.transaction type = "Never"
     */
    public void registerRemoteNoFail(TestRemote remote) throws Exception {
        Context ctx = getOrCreateContextNoFail(TestRemote.CONTEXT_PATH);

        ctx.rebind(TestRemote.JNDI_NAME, remote);
    }

    /**
     * This method if called multiple times will fail on local JNDI lookup. It will try to make a copy of RMI object
     * which extends org.omg.CORBA.portable.ObjectImpl and has transient attribute __delegate. Exception thrown will
     * have message "org.omg.CORBA.BAD_OPERATION: The delegate has not been set!"
     * 
     * @param path - JNDI Path
     * 
     * @return JNDI subcontext context which corresponds to path
     * 
     * @throws Exception
     */
    public Context getOrCreateContext(String path) throws Exception {

        if (path == null) {
            return null;
        }

        // this context will use LocalSerialContextProviderImpl for JNDI lookup and will fail on second call
        Context root = new InitialContext();
        Context parent = root;
        Context leaf = root;

        // Make sure path exists in JNDI tree and if it does not exist then create it
        // Find subcontxt which corresponds to path and return it
        StringTokenizer tokenizer = new StringTokenizer(path, "/");
        while (tokenizer.hasMoreTokens()) {
            String pathElement = tokenizer.nextToken();
            try {
                Object obj = parent.lookup(pathElement);
                leaf = (Context) obj;
            } catch (NameNotFoundException e) {
                try {
                    leaf = parent.createSubcontext(pathElement);
                } catch (NamingException e1) {
                    e1.printStackTrace();
                    break;
                }
            } catch (NamingException e) {
                e.printStackTrace();
                throw e;
            }
            parent = leaf;
        }

        return leaf;
    }

    /**
     * This method if called multiple times will not fail because it does remote JNDI lookup
     * 
     * @param path - JNDI Path
     * 
     * @return JNDI subcontext context which corresponds to path
     * 
     * @throws Exception
     */
    public Context getOrCreateContextNoFail(String path) throws Exception {

        if (path == null) {
            return null;
        }

        Properties props = new Properties();
        // These two properties are making no difference
        // props.put(InitialContext.INITIAL_CONTEXT_FACTORY, "com.sun.enterprise.naming.SerialInitContextFactory");
        // props.put(InitialContext.PROVIDER_URL, "iiop://localhost:3700");

        props.put("org.omg.CORBA.ORBInitialHost", "localhost");
        props.put("org.omg.CORBA.ORBInitialPort", "3700");

        // This context will use RemoteSerialContextProviderImpl and will not fail.
        Context root = new InitialContext(props);
        Context parent = root;
        Context leaf = root;

        // Make sure path exists in JNDI tree and if it does not exist then create it
        // Find subcontxt which corresponds to path and return it
        StringTokenizer tokenizer = new StringTokenizer(path, "/");
        while (tokenizer.hasMoreTokens()) {
            String pathElement = tokenizer.nextToken();
            try {
                Object obj = parent.lookup(pathElement);
                leaf = (Context) obj;
            } catch (NameNotFoundException e) {
                try {
                    leaf = parent.createSubcontext(pathElement);
                } catch (NamingException e1) {
                    e1.printStackTrace();
                    break;
                }
            } catch (NamingException e) {
                e.printStackTrace();
                throw e;
            }
            parent = leaf;
        }

        return leaf;
    }
}